# -*- coding: utf-8 -*-
__version__ = '1.0.0'
__package_name__ = "kong-hmac"
__title__ = 'Kong 网关HMAC鉴权SDK'

__author__ = 'Jager'
__author_email__ = 'im@zhang.ge'

__license__ = 'Apache 2.0'

__python_requires__ = '>=2.6, <4'
__classifiers__ = ["Programming Language :: Python :: 3"]
__install_requires__ = []
__url__ = 'https://git.woa.com/nops/ngate/ngate-sdk/ngate-python.git'

__copyright__ = 'Copyright © zhang.ge. All Rights Reserved.'
__description__ = 'Kong 网关HMAC鉴权SDK'



# -*- coding: utf8 -*-
__author__ = 'Jager'
import sys
import hmac
import hashlib
import base64
import time


class HmacAuth():
    """生成Hmac(hmac-sha256)鉴权头部
    :param hmac_user, String, 可选, 在class初始化时预设HMAC账号，可被函数覆盖
    :param hmac_secret, String, 可选, 在class初始化时预设HMAC密钥，可被函数覆盖
    :return Class Object
    """
    def __init__(self, hmac_user=None, hmac_secret=None):
        self.hmac_user = hmac_user
        self.hmac_secret = hmac_secret
        self.python_version = sys.version_info[0]

    def _sha256_digest(self, content):
        """ sha256计算内容摘要
        :param content, String, 内容
        """
        if self.python_version > 2:
            content_bytes = bytes(content, "utf-8")

        else:
            content_bytes = bytes(content).decode("utf-8")

        content_sha256_digest = hashlib.sha256(content_bytes).digest()
        content_sha256_digest_base64_decode = base64.b64encode(
            content_sha256_digest).decode()
        content_digest = 'SHA-256={0}'.format(
            content_sha256_digest_base64_decode)
        return content_digest

    def _hmac_sha256(self, secret, str_to_sign):
        """生成sha256加密串
        :param secret, String, 指定密钥
        :param str_to_sign, String, 已拼装待签名的数据
        """
        if self.python_version > 2:
            hmac_key = bytes(secret, "utf-8")
            msg_sign = bytes(str_to_sign, "utf-8")

        else:
            hmac_key = bytes(secret)
            msg_sign = bytes(str_to_sign)

        signature = hmac.new(hmac_key, msg_sign,
                             digestmod=hashlib.sha256).digest()
        str_base64 = base64.b64encode(signature).decode()
        return str_base64

    def get_auth_headers(self, hmac_user=None, hmac_secret=None, body=""):
        """获取Hmac鉴权头部
        :param String, 可选, 指定Hmac账号，可以覆盖class预设的Hmac账号
        :param String, 可选, 指定hmac密钥，可以覆盖class预设的Hmac密钥
        :param String, 可选，指定请求Body内容，当网关要求验签body的时候必传，Get请求则传入空值
        :param Dict, 返回Hmac认证头部字典
        """
        if not hmac_user:
            hmac_user = self.hmac_user

        if not hmac_secret:
            hmac_secret = self.hmac_secret

        # 生成body的sha256加密串
        body_digest = self._sha256_digest(body)

        # 生成当前GMT时间，注意格式不能改变，必须形如：Wed, 14 Aug 2019 09:09:28 GMT
        gm_time = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime())

        # 拼装待签名的数据
        str_to_sign = "date: {0}\ndigest: {1}".format(gm_time, body_digest)

        # 生成签名
        signature = self._hmac_sha256(hmac_secret, str_to_sign)

        # 拼装headers
        headers = {}
        headers["Authorization"] = (
            'hmac username="{0}", algorithm="hmac-sha256", headers="date digest",'
            'signature="{1}"'.format(hmac_user, signature))
        headers["Digest"] = body_digest
        headers["Date"] = gm_time
        return headers



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from setuptools import find_packages, setup

settings = dict()

root_path = os.path.dirname(os.path.realpath(__file__))

with open("{0}/kong_hmac/__version__.py".format(root_path), "rb") as version:
    setup_info = version.read()
    exec(setup_info)

with open("{0}/README.rst".format(root_path), "rb") as file_readme:
    readme = file_readme.read().decode("utf-8")

settings.update(name=__package_name__,
                version=__version__,
                description=__title__,
                long_description=readme,
                author=__author__,
                author_email=__author_email__,
                license=__license__,
                url=__url__,
                packages=find_packages(),
                zip_safe=False,
                classifiers=__classifiers__)

setup(**settings)



# # -*- coding: utf-8 -*-
import os
import sys
# 兼容安装到非系统默认目录导入找不到模块的问题
package_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
if package_path not in sys.path:
    sys.path.append(package_path)

from .__version__ import __title__, __description__, __url__, __version__
from .__version__ import __author__, __author_email__, __license__, __copyright__
from .hmac_auth import HmacAuth  # pylint: disable=unused-import
